//
//  BoardView.m
//  GameOfLife
//
//  Created by Grant Limberg, Kevin Toste on 4/13/09.
//

#import "BoardView.h"
#import "Game.h"
#import "Board.h"
#import "Square.h"
#import "Player.h"
#import "Car.h"

@implementation BoardView

- (id)initWithFrame:(NSRect)frame {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
    if (self = [super initWithFrame:frame]) {
        // load the board image from the bundle resources folder
		boardImage = [[NSImage alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"board" ofType:@"jpg"]];
		boardSize = [boardImage size];
		
		// make the image larger
		boardSize.width *= 1.4;
		boardSize.height *= 1.4;
		[boardImage setSize:boardSize];
		
		NSRect rect;
		rect.size = boardSize;
		// set the frame size to the image size
		[self setFrame:rect];
		theGame = [Game getInstance];
		theBoard = [theGame board];
		
		// listen for RefreshDisplay notifications
		[[NSNotificationCenter defaultCenter] addObserver:self 
												 selector:@selector(refreshBoard:) 
													 name:@"RefreshDisplay" 
												   object:nil];
    }
    return self;
}

- (void)awakeFromNib {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	NSRect rect = [self frame];
	NSRect myRect = NSMakeRect(600, 300, rect.size.width, rect.size.height);
	[self scrollRectToVisible:myRect];
	
}

- (void)drawRect:(NSRect)rect {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
    // Draw the board to the BoardView portion of the window
	[boardImage drawInRect:rect 
				  fromRect:rect 
				 operation:NSCompositeSourceOver 
				  fraction:1.0];
	
	// TODO: Draw cars
	NSArray *players = [theGame players];
	for( Player *p in players ) {
		Car *c = [p car];
		NSImage *img = [c carImage];
		Square *square = [p position];
		NSPoint pos = [square position];

		[img drawAtPoint:pos 
				fromRect:NSZeroRect 
			   operation:NSCompositeSourceOver 
				fraction:1.0];
	}
}

- (void)refreshBoard:(NSNotification*)aNotification {
	[self setNeedsDisplay:YES];
}

@end
