//
//  Game.m
//  GameOfLife
//
//  Created by Grant Limberg, Kevin Toste on 4/18/09.
//

#import "Game.h"
#import "Board.h"
#import "Deck.h"
#import "Player.h"
#import "Card.h"
#import "Turn.h"

static Game *theGame = nil;

@implementation Game

@synthesize players;

+ (Game*)getInstance {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	@synchronized(self) {
		if( theGame == nil ) {
			[[Game alloc] init];
		}
	}
	return theGame;
}

- (id)init {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	@synchronized(self) {
		if( self = [super init] ) {
			players = [[NSMutableArray alloc] init];
			theBoard = [[Board alloc] init];
			careerDeck = [Deck getCareerDeck];
			stockDeck = [Deck getStockDeck];
			salaryDeck = [Deck getSalaryDeck];
			houseDeck = [Deck getHouseDeck];
			tilePile = [Deck getTilePile];

			[[NSNotificationCenter defaultCenter] addObserver:self 
													 selector:@selector(startNextTurn:) 
														 name:@"TurnDidFinish" 
													   object:nil];
			
			theGame = self;
		}
	}
	return theGame;
}

- (void)addPlayer:(Player*)aPlayer {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[players addObject:aPlayer];
}

- (void)removePlayer:(Player*)aPlayer {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[players removeObject:aPlayer];
}

- (NSInteger)numberOfPlayers {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [players count];
}

- (void)start {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[[NSNotificationCenter defaultCenter] postNotificationName:@"SelectFirstPlayer" 
														object:@"SelectFirstPlayer"];
	currentTurn = [[Turn alloc] init];

	[currentTurn doTurn];
}

- (void)startNextTurn:(NSNotification*)aNotification {
	debug(@"%s:%d", __FUNCTION__, __LINE__);	
	if( [currentTurn hasNextTurn] ) {
		currentTurn = [currentTurn getNextTurn];
		[currentTurn doTurn];
	} else {
		// end game
	}
}

- (Card*)lifeTile {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [tilePile getCard];
}

- (Card*)salaryCard {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [salaryDeck getCard];
}

- (void)replaceSalaryCard:(Card*)aCard {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[salaryDeck replaceCard:aCard];
}

- (Card*)stockCard {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [stockDeck getCard];
}

- (void)replaceStockCard:(Card*)aCard {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[stockDeck replaceCard:aCard];
}

- (Card*)careerCard {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [careerDeck getCard];
}

- (void)replaceCareerCard:(Card*)aCard {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[careerDeck replaceCard:aCard];
}

- (Card*)houseCard {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [houseDeck getCard];
}

- (void)replaceHouseCard:(Card*)aCard {
	[houseDeck replaceCard:aCard];
}

- (Board*)board {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return theBoard;
}

#pragma mark Overriden Methods of NSObject
+ (id)allocWithZone:(NSZone *)zone
{
    @synchronized(self) {
        if (theGame == nil) {
            theGame = [super allocWithZone:zone];
            return theGame;  // assignment and return on first allocation
        }
    }
    return nil; //on subsequent allocation attempts return nil
}

- (id)copyWithZone:(NSZone *)zone
{
    return self;
}

- (id)retain
{
    return self;
}

- (unsigned)retainCount
{
    return UINT_MAX;  //denotes an object that cannot be released
}

- (void)release
{
    //do nothing
}

- (id)autorelease
{
    return self;
}

@end
