//
//  GameController.m
//  GameOfLife
//
//  Created by Grant Limberg, Kevin Toste on 4/11/09.
//

#import "GameViewController.h"

#import "Player.h"
#import "PlayerTableDataSource.h"
#import "Game.h"
#import "Car.h"
#import "Board.h"
#import "Square.h"
#import "BoardView.h"
#import "HouseInsurance.h"
#import "CarInsurance.h"
#import "House.h"

@implementation GameViewController

- (id)init {
	if( self = [super init] ) {
		isStarted = NO;
		addedPlayers = 0;
		playerCount = 0;
		theGame = [Game getInstance];

		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(showCareerSheet:)
													 name:@"ShowCareerSheet"
												   object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(showHouseSheet:)
													 name:@"ShowHouseSheet"
												   object:nil];
		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(showSwapSalrySheet:)
													 name:@"ShowSwapSalarySheet"
												   object:nil];
	}
	return self;
}

- (IBAction)showNewGameSheet:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[self resetGame];

	[NSApp beginSheet:newGameSheet 
	   modalForWindow:mainWindow 
		modalDelegate:nil
	   didEndSelector:NULL
		  contextInfo:NULL];
}

- (IBAction)newGameSheetCancelButtonPressed:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[self closeNewGameSheet:sender];
	[self resetGame];
}

- (IBAction)newGameSheetOKButtonPressed:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[self closeNewGameSheet:sender];
	NSMenuItem *selectedItem = [numPlayers selectedItem];
	playerCount = [[selectedItem title] intValue];
	[self showAddPlayerSheet:sender];
}

- (void)closeNewGameSheet:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[NSApp endSheet:newGameSheet];
	[newGameSheet orderOut:sender];
}

- (IBAction)showAddPlayerSheet:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[NSApp beginSheet:addPlayerSheet
	   modalForWindow:mainWindow 
		modalDelegate:nil
	   didEndSelector:NULL
		  contextInfo:NULL];
}

- (IBAction)addPlayerSheetOKButtonPressed:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	
	NSString *newPlayerName = [curPlayerName stringValue];
	[curPlayerName setStringValue:@""];
	
	// create new player
	++addedPlayers;
	Player *aPlayer = [[Player alloc] 
					   initWithName:newPlayerName 
					   andMoney:0];
	
	BOOL shouldGoToCollege = NO;
	if( [goToCollege state] == NSOnState ) {
		shouldGoToCollege = YES;
		[aPlayer setStartPosition:[[theGame board] startCollegeSquare]];
	} else {
		[aPlayer setStartPosition:[[theGame board] startCareerSquare]];
	}
	
	[goToCollege setState:NSOffState];
	
	[self closeAddPlayerSheet:sender];
	
	[aPlayer setGoToCollege:shouldGoToCollege];
	
	Car *car = [[Car alloc] init];
	NSButtonCell *selectedCell = [carButtons selectedCell];
	[car setColor:[selectedCell alternateTitle]];
	[selectedCell setEnabled:NO];
	[aPlayer setCar:car];
	
	// add new player to game
	[self addPlayer:aPlayer];
	
	// tell view to update
	[[NSNotificationCenter defaultCenter] postNotificationName:@"RefreshDisplay" 
														object:@"RefreshDisplay"];
	
	if( addedPlayers < playerCount ) {
		[self showAddPlayerSheet:sender];
	} else {
		[self beginGame];
	}
}

- (IBAction)addPlayerSheetCancelButtonPressed:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[self closeAddPlayerSheet:sender];
	[self resetGame];
}

- (void)closeAddPlayerSheet:(id)sender {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[NSApp endSheet:addPlayerSheet];
	[addPlayerSheet orderOut:sender];
}

- (void)beginGame {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[theGame start];
}

- (void)addPlayer:(Player*)aPlayer {
	[theGame addPlayer:aPlayer];
	[playerTable reloadData];
}

- (void)resetGame {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	
	playerCount = 0;
	addedPlayers = 0;
	[[theGame players] removeAllObjects];
	[playerTable reloadData];
	NSArray *cells = [carButtons cells];
	for( NSButtonCell *c in cells ) {
		[c setEnabled:YES];
	}
}

- (void)showCareerSheet:(NSNotification*)aNotification {
	NSDictionary *userInfo = [aNotification userInfo];
	
	[careerName setStringValue:[userInfo objectForKey:@"CareerName"]];
	[salaryAmount setStringValue:[userInfo objectForKey:@"SalaryAmount"]];
	[taxAmount setStringValue:[userInfo objectForKey:@"TaxAmount"]];
	[playerName setStringValue:[userInfo objectForKey:@"PlayerName"]];

	[NSApp beginSheet:careerSheet
	   modalForWindow:mainWindow
		modalDelegate:nil
	   didEndSelector:nil
		  contextInfo:nil];
	[NSApp runModalForWindow:careerSheet];

	[NSApp endSheet:careerSheet];
	[careerSheet orderOut:self];
}

- (IBAction)closeCareerSheet:(id)sender {
	[NSApp stopModal];
}

- (void)showHouseSheet:(NSNotification*)aNotification {
	NSDictionary *userInfo = [aNotification userInfo];

	[housePlayerName setStringValue:[userInfo objectForKey:@"PlayerName"]];
	NSString *houseString = [[NSString alloc] initWithFormat:@"%@.\n\n%@", 
									 [[userInfo objectForKey:@"HouseCard"] name],
									 [[userInfo objectForKey:@"HouseCard"] description]];

	[houseDescription setStringValue:houseString];
	
	NSString *price = [[NSString alloc] initWithFormat:@"%d",
									[[userInfo objectForKey:@"HouseCard"] cost]];
	[housePrice setStringValue:price];

	NSString *insuranceCost = [[NSString alloc] initWithFormat:@"%d",
									   [[userInfo objectForKey:@"HouseCard"] insuranceCost]];
	[houseInsurance setStringValue:insuranceCost];

	[NSApp beginSheet:houseSheet
	   modalForWindow:mainWindow
		modalDelegate:nil
	   didEndSelector:nil
		  contextInfo:nil];
	[NSApp runModalForWindow:houseSheet];

	[NSApp endSheet:houseSheet];
	[houseSheet orderOut:self];
}

- (IBAction)closeHouseSheet:(id)sender {
	[NSApp stopModal];
}

- (IBAction)buyHouseInsurance:(id)sender {
	Player *player = [playerTable currentPlayer];
	if( ![player hasHouseInsurance] ) {
		HouseInsurance *houseIns = [[HouseInsurance alloc] init];
		[player addInsurance:houseIns];

		House *playerHouse = [player house];
		NSInteger money = [player money];
		money -= [playerHouse insurancePrice];
		[player setMoney:money];
	}
}

- (IBAction)showInsuranceSheet:(id)sender {
	[NSApp beginSheet:insuranceSheet
	   modalForWindow:mainWindow
		modalDelegate:nil
	   didEndSelector:nil
		  contextInfo:nil];
	[NSApp runModalForWindow:insuranceSheet];

	[NSApp endSheet:insuranceSheet];
	[insuranceSheet orderOut:self];
}

- (IBAction)closeInsuranceSheet:(id)sender {
	[NSApp stopModal];
}

- (IBAction)buyCarInsurance:(id)sender {
	Player *player = [playerTable currentPlayer];
	if( ![player hasCarInsurance] ) {
		CarInsurance *carIns = [[CarInsurance alloc] init];
		[player addInsurance:carIns];

		NSInteger money = [player money];
		money -= 10000;
		[player setMoney:money];
	}
}

- (IBAction)buyStock:(id)sender {
	Player *player = [playerTable currentPlayer];
	if( [player stockNumber] == -1 && [player money] >= 50000 ) {
		[player giveCard:[theGame stockCard]];
	}
}

- (IBAction)showBankSheet:(id)sender {
	[NSApp beginSheet:bankSheet
	   modalForWindow:mainWindow
		modalDelegate:nil
	   didEndSelector:nil
		  contextInfo:nil];
	[NSApp runModalForWindow:bankSheet];
	[NSApp endSheet:bankSheet];
	[bankSheet orderOut:self];
}

- (IBAction)closeBankSheet:(id)sender {
	[NSApp stopModal];
}

- (IBAction)showSwapSalrySheet:(NSNotification*)aNotification {
	[swapButton setEnabled:YES];
	[closeButton setEnabled:NO];

	[NSApp beginSheet:swapSheet
	   modalForWindow:mainWindow
		modalDelegate:nil
	   didEndSelector:nil
		  contextInfo:nil];
	[NSApp runModalForWindow:swapSheet];
	[NSApp endSheet:swapSheet];
	[swapSheet orderOut:self];
}

- (IBAction)closeSwapSalarySheet:(id)sender {
	[NSApp stopModal];
}

@end
