//
//  Player.m
//  GameOfLife
//
//  Created by Grant Limberg, Kevin Toste on 3/9/09.
//

#import "Player.h"
#import "Career.h"
#import "CarInsurance.h"
#import "HouseInsurance.h"
#import "Card.h"
#import "Car.h"
#import "Loan.h"
#import "Square.h"
#import "Child.h"
#import "House.h"
#import "StopSquare.h"

@implementation Player

@synthesize career;
@synthesize name;
@synthesize money;
@synthesize goToCollege;
@synthesize car;
@synthesize sex;
@synthesize married;
@synthesize loans;
@synthesize position;
@synthesize isFirstTurn;
@synthesize house;
@synthesize stockNumber;
@synthesize collectedLifeTiles;

- (id)initWithName:(NSString *)aName andMoney:(NSInteger)someMoney {
	if( self = [super init] ) {
		name = aName;
		money = someMoney;
		goToCollege = NO;
		insurance = [[NSMutableArray alloc] init];
		children = [[NSMutableArray alloc] init];
		married = NO;
		loans = [[NSMutableArray alloc] init];
		position = nil;
		isFirstTurn = YES;
		collectedLifeTiles = [[NSMutableArray alloc] init];
		stockNumber = -1;
	}
	return self;
}

- (NSInteger)getNetWorth {
	return money;
}

- (void)addInsurance:(Insurance*)anInsurancePolicy {
	[insurance addObject:anInsurancePolicy];
}

- (BOOL)hasCarInsurance {
	for( Insurance *i in insurance ) {
		if( [i isMemberOfClass:[CarInsurance class]] )
			return YES;
	}
	return NO;
}

- (BOOL)hasHouseInsurance {
	for( Insurance *i in insurance ) {
		if( [i isMemberOfClass:[HouseInsurance class]] )
			return YES;
	}
	return NO;
}

- (void)giveCard:(Card*)aCard {
	[aCard doCardActionToPlayer:self];
}

- (void)addLoan:(Loan*)aLoan {
	money += [aLoan amount];
	[loans addObject:aLoan];
}

- (void)payLoan:(Loan*)aLoan {
	if( money - [aLoan paybackAmount] >= 0 ) {
		money -= [aLoan paybackAmount];
		[loans removeObject:aLoan];
	} else {
		NSAlert *alert = [NSAlert alertWithError:@"You don't have enough money to pay back your loan yet!"];
		[alert runModal];
	}
}

- (BOOL)hasLoans {
	if( [loans count] > 0 )
		return YES;
	else
		return NO;
}

- (Square*)position {
	return position;
}

- (void)setPosition:(Square*)aSquare {
	[position setOccupied:NO];
	
	if( ![aSquare isMemberOfClass:[StopSquare class]] ) {
		while( [aSquare occupied] ) {
			aSquare = [aSquare nextSquare];
		}
		[aSquare setOccupied:YES];
	}
	position = aSquare;
}

- (void)setStartPosition:(Square*)aSquare {
	position = aSquare;
}

- (void)addChild:(Child*)aChild {
	[children addObject:aChild];
}

- (NSInteger)children {
	return [children count];
}

- (void)setRetired:(BOOL)ret {
	retired = ret;
}

- (BOOL)isRetired {
	return retired;
}

- (void)giveLifeTile:(Card*)aCard {
	[collectedLifeTiles addObject:aCard];
}

@end
