//
//  PlayerInfoView.m
//  GameOfLife
//
//  Created by Grant Limberg, Kevin Toste on 3/9/09.
//

#import "PlayerInfoCell.h"
#import "Player.h"
#import "Career.h"

@implementation PlayerInfoCell

@synthesize player;

- (id)initWithPlayer:(Player*)aPlayer {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(self = [super initTextCell:[aPlayer name]]) {
		player = aPlayer;
	}
	return self;
}

- (NSTextFieldBezelStyle)bezelStyle {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return NSRoundedBezelStyle;
}

- (NSCell*)copyWithZone:(NSZone *)zone {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	PlayerInfoCell *cell = (PlayerInfoCell *)[super copyWithZone:zone];
    return cell;
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView {
	debug(@"%s:%d", __FUNCTION__, __LINE__);

	[self setTextColor:[NSColor blackColor]];
		
	NSColor* primaryColor   = [self isHighlighted] ? [NSColor alternateSelectedControlTextColor] : [NSColor textColor];
	
	NSString* playerName = [player name];
	NSString* spacing = @"\t\t\t\t";
	NSString* careerName;
	if( [player career] == nil )
		careerName = @"";
	else
		careerName = [[player career] careerName];
	
	NSString* primaryText   = [[playerName stringByAppendingString:spacing] stringByAppendingString:careerName]; 
	
	NSDictionary* primaryTextAttributes = [NSDictionary dictionaryWithObjectsAndKeys: primaryColor, NSForegroundColorAttributeName,
										   [NSFont systemFontOfSize:13], NSFontAttributeName, nil];	
	[primaryText drawAtPoint:NSMakePoint(cellFrame.origin.x+10, cellFrame.origin.y) withAttributes:primaryTextAttributes];
	
	NSColor* secondaryColor = [self isHighlighted] ? [NSColor alternateSelectedControlTextColor] : [NSColor disabledControlTextColor];
	
	NSInteger money = [player getNetWorth];
	NSMutableString *moneyString = [[NSString stringWithFormat:@"%d", money] mutableCopy];
	NSInteger length = [moneyString length];
	int pos = length;
	pos -= 3;
	while( pos > 0 ) {
		[moneyString insertString:@"," atIndex:pos];
		pos -= 3;
	}
	
	NSString* secondaryText = [@"$" stringByAppendingString:[moneyString stringByAppendingString:@".00"]];
	NSDictionary* secondaryTextAttributes = [NSDictionary dictionaryWithObjectsAndKeys: secondaryColor, NSForegroundColorAttributeName,
											 [NSFont systemFontOfSize:10], NSFontAttributeName, nil];	
	[secondaryText drawAtPoint:NSMakePoint(cellFrame.origin.x+10, cellFrame.origin.y+cellFrame.size.height/2) 
				withAttributes:secondaryTextAttributes];
	
	
	[[NSGraphicsContext currentContext] saveGraphicsState];
	float yOffset = cellFrame.origin.y;
	if ([controlView isFlipped]) {
		NSAffineTransform* xform = [NSAffineTransform transform];
		[xform translateXBy:0.0 yBy: cellFrame.size.height];
		[xform scaleXBy:1.0 yBy:-1.0];
		[xform concat];		
		yOffset = 0-cellFrame.origin.y;
	}	
	
	NSImageInterpolation interpolation = [[NSGraphicsContext currentContext] imageInterpolation];
	[[NSGraphicsContext currentContext] setImageInterpolation: NSImageInterpolationHigh];	
	
	[[NSGraphicsContext currentContext] setImageInterpolation: interpolation];
	
	[[NSGraphicsContext currentContext] restoreGraphicsState];	
}

@end
