//
//  PlayerTableDataSource.m
//  GameOfLife
//
//  Created by Grant Limberg, Kevin Toste on 3/14/09.
//

#import "PlayerTableDataSource.h"
#import "PlayerInfoCell.h"
#import "Player.h"
#import "Career.h"
#import "Game.h"

@implementation PlayerTableDataSource

- (id)init {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if(self = [super init]) {
		theGame = [Game getInstance];
		[[NSNotificationCenter defaultCenter] addObserver:self 
												 selector:@selector(selectFirstPlayer:) 
													 name:@"SelectFirstPlayer" 
												   object:nil];

		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(selectNextPlayer:)
													 name:@"SelectNextPlayer"
												   object:nil];

		[[NSNotificationCenter defaultCenter] addObserver:self
												 selector:@selector(reloadData:)
													 name:@"RefreshDisplay"
												   object:nil];
		curPlayer = -1;
	}
	return self;
}

- (void) awakeFromNib {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
}

- (NSInteger)numberOfRowsInTableView:(NSTableView *)aTableView {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [[theGame players] count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn 
			row:(int)rowIndex {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	return [[[theGame players] objectAtIndex:rowIndex] name];
}

- (void)tableView:(NSTableView *)aTableView 
  willDisplayCell:(id)theCell 
   forTableColumn:(NSTableColumn *)aTableColumn 
			  row:(NSInteger)rowIndex {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	[theCell setPlayer:[[theGame players] objectAtIndex:rowIndex]];
}

- (void)reloadData {
	[playerTable reloadData];
}

- (void)reloadData:(NSNotification*)aNotification {
	[playerTable reloadData];
}

- (void)selectFirstPlayer:(NSNotification*)aNotification {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	if( [playerTable selectedRow] == -1 ) {
		curPlayer = 0;
		NSIndexSet *index = [[NSIndexSet alloc] initWithIndex:curPlayer];
		
		[playerTable selectRowIndexes:index byExtendingSelection:NO];
	}
}

- (void)selectNextPlayer:(NSNotification*)aNotification {
    curPlayer = [playerTable selectedRow];
    if( curPlayer == [[theGame players] count]-1 ) {
        curPlayer = 0;
    } else {
        curPlayer +=1;
    }
    
    NSIndexSet *newIndex = [[NSIndexSet alloc] initWithIndex:curPlayer];
    [playerTable selectRowIndexes:newIndex byExtendingSelection:NO];
}	

- (Player*)currentPlayer {
	if( curPlayer >= 0 )
		return [[theGame players] objectAtIndex:curPlayer];
	else
		return nil;
}

@end
