//
//  TurnThread.m
//  GameOfLife
//
//  Created by Grant Limberg, Kevin Toste on 5/7/09.
//

#import "TurnThread.h"
#import "Player.h"
#import "Game.h"
#import "Square.h"
#import "SpinAgainSquare.h"
#import "StopSquare.h"
#import "PayDay.h"
#import "RetirementSquare.h"

@implementation TurnThread

- (id)init {
	debug(@"%s:%d", __FUNCTION__, __LINE__);	
	if( self = [super init] ) {
		theGame = [Game getInstance];
		spinFinished = NO;

		[[NSNotificationCenter defaultCenter] addObserver:self 
												 selector:@selector(setSpinFinished:) 
													 name:@"SpinFinished" 
												   object:nil];
	}
	return self;
}

- (void)main {
	debug(@"%s:%d", __FUNCTION__, __LINE__);	
	
	NSMutableArray *players = [theGame players];

	for( Player *p in players ) {
		if( ![p isRetired] ) {
			[self doTurn:p];
		} // else no turn

		[[NSNotificationCenter defaultCenter] postNotificationName:@"SelectNextPlayer"
															object:self];
	}

	[[NSNotificationCenter defaultCenter] postNotificationName:@"TurnDidFinish"
														object:self];
}

- (void)doTurn:(Player*)p {
	debug(@"%s:%d", __FUNCTION__, __LINE__);
	
	Square *square = [p position];
	// if it's the player's first turn, perform the action of the first square
	// before they spin.
	if( [p isFirstTurn] ) {
		[p setIsFirstTurn:NO];
		[square performSquareAction:p];
	}

	// enable spin button
	[[NSNotificationCenter defaultCenter] postNotificationName:@"EnableSpinButton" 
														object:self];

	// wait for spin to be finished
	while( !spinFinished ) {
		[NSThread sleepForTimeInterval:0.1];
	}

	[[NSNotificationCenter defaultCenter] postNotificationName:@"DisableSpinButton" 
														object:self];
	int count;
	for( count = 0; count < spunNumber; ++count ) {
		square = [square nextSquare];
		
		// do payday if we're passing a payday square
		if( [square isMemberOfClass:[PayDay class]] ) {
			[square performSquareAction:p];
		}
		
		[[NSNotificationCenter defaultCenter] postNotificationName:@"RefreshDisplay"
														object:self];
		[p setPosition:square];

		[NSThread sleepForTimeInterval:0.5];

		// if we're on a stop square, stop moving forward.
		if( [square isMemberOfClass:[StopSquare class]] ||
			[square isMemberOfClass:[RetirementSquare class]] ) {
			break;
		}
	}

	[p setPosition:square];
	[[NSNotificationCenter defaultCenter] postNotificationName:@"RefreshDisplay"
														object:self];

	// since we've already processed the payday, skip it if we end up landing on it
	if( ![square isMemberOfClass:[PayDay class]] ) {
		[square performSquareAction:p];
	}
	spinFinished = NO;

	// if we're on a spin again square or a stop square, spin again
	if( [square isMemberOfClass:[SpinAgainSquare class]] || 
		( [square isMemberOfClass:[StopSquare class]] && count < spunNumber)  )
	{
		[self doTurn:p];
	}
}

- (void)setSpinFinished:(NSNotification*)aNotification {
	debug(@"%s:%d", __FUNCTION__, __LINE__);	
	spinFinished = YES;
	NSNumber *num = [[aNotification userInfo] objectForKey:@"number"];

	spunNumber = [num intValue];

	NSArray *players = [theGame players];
	for( Player *p in players ) {
		if( [p stockNumber] == spunNumber ) {
			NSInteger money = [p money];
			money += 10000;
			[p setMoney:money];
		}
	}
}

@end
